/**
 * @author Martin Heusel
 * 
 * Copyright (C) 2008 Martin Heusel (mhe@bioinf.jku.at),
 *                      
 * Johannes Kepler University, Linz, Austria
 * Institute of Bioinformatics.
 * The software is maintained by Martin Heusel.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * If you use this library, please cite:
 *
 * @article{SeppHochreiter07152007,
 *   author = {Hochreiter, Sepp and Heusel, Martin and Obermayer, Klaus},
 *   title = {{Fast Model-based Protein Homology Detection without Alignment}},
 *   journal = {Bioinformatics},
 *   volume = {23},
 *   number = {14},
 *   pages = {1728-1736},
 *   doi = {doi:10.1093/bioinformatics/btm247},
 *   year = {2007},
 *   URL = {http://bioinformatics.oxfordjournals.org/cgi/content/abstract/btm247v1},
 *   eprint = {http://bioinformatics.oxfordjournals.org/cgi/reprint/btm247v1}
 * }
 * 
 * $Id: ComputeRoc.java 225 2009-02-12 19:13:05Z mhe $
 *
 */


package at.jku.bioinf.jlstm;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/**
 * Computes the ROC
 * 
 * @author mhe
 *
 */
public class ComputeRoc {
    
    double auc;
    double aucn;
    
    static class RocListComparator implements Comparator<RocData> {
    
    /**
     * Comparator for two roc datas 
     * 
     * (non-Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(RocData rd1, RocData rd2) {
        
        if (rd1.getVal() > rd2.getVal()) {
            return 1;
        }
        else if (rd1.getVal() < rd2.getVal()) {
            return -1;
        }
        
        return 0;

      }
    
    }
    
    /**
     * Compute AUC
     * 
     * @param rocList
     * @param rp
     * @param rn
     * @param rocn
     */
    public void computeAUC (ArrayList<RocData> rocList, int rp, int rn, int rocn) {

        int tp = 0;
        int fp = 0;

        double[] xarr = new double[rp + rn];
        double[] yarr = new double[rp + rn];

        double[] xarrn = new double[rp + rn];
        
        RocListComparator rcl = new RocListComparator();

        Collections.sort(rocList, rcl);
        Collections.reverse(rocList);
        
        // Counter for ROCN
        int countn = 0;
        
        for (int i = 0; i < rocList.size(); i++) {
            
            RocData rd = rocList.get(i);

            if (rd.getLabel() != 0) {
                tp++;
            }
            else {
                fp++;
            }

            xarr[i] = (double) fp / (double) rn;
            yarr[i] = (double) tp / (double) rp;
            
            
            if (fp <= rocn) {
                xarrn[i] = (double) fp / (double) rocn;
                countn++;
            }

            auc    = 0;
            aucn   = 0;
            double prevx  = 0;
            double prevxn = 0;

            for (int j = 0; j < xarr.length; j++) {

                if (xarr[j] != prevx) {

                    auc += (xarr[j] - prevx) * yarr[j];
                    prevx = xarr[j];

                }

                if (j < countn) {

                    if (xarrn[j] != prevxn) {

                        aucn += (xarrn[j] - prevxn) * yarr[j];
                        prevxn = xarrn[j];

                    }
                    
                }

            }

        }
        
        rocList.clear();

    }

    public double getAuc() {
        return auc;
    }

    public double getAucn() {
        return aucn;
    }
}
